/* ***** BEGIN LICENSE BLOCK *****
 
 
 Copyright 2012 Adam Skillen (a_skil(at)encs.concordia.ca), Mohammad Mannan (mmannan(at)encs.concordia.ca)
 
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.  Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.  Neither the name of Carleton
 * University nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written
 * permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 * ***** END LICENSE BLOCK ***** */

  /**
  * 09-MAY-12
  * 	TODO:  0
  * 	FIXME: 0
  * 	NOTE:  0
  */ 

var EXPORTED_SYMBOLS = [];

const Cc = Components.classes;
const Ci = Components.interfaces;

Components.utils.import("resource://myphrase/common.js");
Components.utils.import("resource://myphrase/encoder.js");  
Components.utils.import("resource://myphrase/commonWords.js");

  MyphraseCCSL.ParseText = {
	prefs: null,
	filePaths: [],		//sent messages
	userDict: [],		//words from sent mail
	userFreqs: [],		//word frequencies
	outWords: "",		//string of words to write to local file
	parent: null,
    CC: MyphraseCCSL.words.CC,	// Coord Conjuncn
    DT: MyphraseCCSL.words.DT,	// Determiner
    EX: MyphraseCCSL.words.EX,	// Existential there
    IN: MyphraseCCSL.words.IN,	// Preposition
    JJ: [],
    MD: MyphraseCCSL.words.MD,	// Modal
    NN: [],
    PRP$: MyphraseCCSL.words.PRP$,	// Possessive pronoun
    PRP: MyphraseCCSL.words.PRP,	// Personal pronoun
    RB: [],	// Adverb
    RP: MyphraseCCSL.words.RP,	// Particle
    TO: MyphraseCCSL.words.TO,// to
    UH: MyphraseCCSL.words.UH,	// Interjection
    VB: [],	// verb, base form			(eat)
    WP: MyphraseCCSL.words.WP,	//  Wh pronounwho,what
	
	//  build a custom dictionary from words in text files
	buildDict: function(parent) {
	
		this.prefs = null;
		this.filePaths = [];		
		this.userDict = [];	
		this.userFreqs = [];	
		this.outWords = "";
		this.parent = null;
	
	
		Components.utils.import("resource://gre/modules/FileUtils.jsm");  
		Components.utils.import("resource://gre/modules/NetUtil.jsm");  
		
		this.prefs = Components.classes["@mozilla.org/preferences-service;1"]
				.getService(Components.interfaces.nsIPrefService)
				.getBranch("extensions.myphrase.");
	
		this.parent = parent;
	
		// get text files
		var files = this.pickTextFiles(parent);
		while (files.hasMoreElements()) {
			        var arg = files.getNext().QueryInterface(Components.interfaces.nsILocalFile).path;
					this.filePaths.push(arg);
		}
		
		// parse and lex sent messages
		this.parseAndLex();
		
		// remove common words
		this.cullCommon();
		
		// count word frequencies and keep only most common words
		this.countAndTrucFreqs();
		
		// prompt user to save file
		return this.saveDictFile(parent);
	},

	parseAndLex: function () {

		// user's max word length preference
		var maxWordLen = this.prefs.getIntPref("wordLengthPref");

		// REGEX to remove html tags, digits, punctuation, etc...
		for (var i = 0; i < this.filePaths.length; i++) {
		
		  var text = MyphraseCCSL.Common.GetFileContent("file://" + this.filePaths[i]);
		  text = unescape(text);										//unescape encoded characters
		  text = MyphraseCCSL.Encoder.htmlDecode(text);					//decode encoded characters
		  text = text.replace(/(<br>|<br\/>|<br \/>).*?/ig,"\r\n");		// replace <br> tags with line-breaks
		  
		  text = text.replace(/(-\s+)/g, '');						// concatenate hyphenated words spanning lines
		  text = text.replace (/[-]+/ig, ' ');							// split compound words on hyphen
										
		  text = text.replace(/<(.|\n)*?>/g," ");						// remove all open and close HTML tags
		  text = text.replace (/[^a-zA-Z '\r\n]+/ig, ' ');				// keep only letters, space, and linefeeds (and apostrophe for now)
		  text = text.replace(/\s{2,}/g," ");							// truncate whitespace to single space
		  var words = text.split(/\s/);									// split text on whitespace or linebreaks
		  
		  // insert words if they are the right length
		  for (var j = 0; j < words.length; j++) {
			words[j] = words[j].replace(/['].*/g,"");		// remove apostorphe and everything after it
			if (words[j].length <= maxWordLen) { // && words[j].length > 1) {
			  var k = this.userDict.indexOf(words[j].toLowerCase());
			  if (k > -1) {
				this.userFreqs[k] += 1;	  // track frequencies
			  } else {
				this.userDict.push(words[j].toLowerCase());
				this.userFreqs.push(1);
			  }
			}
		  }
		}
		return;
	},
	
	cullCommon: function () {	
			var freqWords = [];
			freqWords = freqWords.concat(this.CC, this.DT, this.EX, this.IN, this.MD, this.PRP$, this.PRP, this.RP, this.TO, this.UH, this.WP);

            			
			for (var i = 0; i < freqWords.length; i++) {
				var k = this.userDict.indexOf(freqWords[i].toLowerCase());
				if (k > -1) {
				  this.userFreqs[k] = -1;
				}
			}
			
	},
	
	countAndTrucFreqs: function () {

		//user's dictionary size preference
		var dictSize = this.prefs.getIntPref("dictSizePref");

		// "associative array" and sort function adapted from examples at: 
		// http://stackoverflow.com/questions/5199901/how-to-sort-an-associative-array-by-its-values-in-javascript
		var words = [];
		for (var i = 0; i < this.userDict.length; i++) {
			if (this.userFreqs[i] > 0) {
				words.push([this.userDict[i], this.userFreqs[i]]);
			}
		}
		// sort user's dictionary based on word frequency
		words.sort(function(a, b) {
			a = a[1];
			b = b[1];

			return a < b ? 1 : (a > b ? -1 : 0);
		});

		// keep user's most frequent words
		var stop = words.length > dictSize ? dictSize : words.length;
		for (var i = 0; i < stop; i++) {
			if (words[i][0].length > 0)
				this.outWords += words[i][0] + "\r\n";
		}
		
		// trim trailing whitespace
		this.outWords = this.outWords.replace(/\s*$/, "");
	},
	
	saveDictFile: function (parent) {
		// save the file containing the adjusted dictionary words separated by newlines
		// adapted from examples on MDN: https://developer.mozilla.org/en/Code_snippets/File_I%2F%2FO
		// https://developer.mozilla.org/en/nsIFilePicker#Example_2
		const nsIFilePicker = Components.interfaces.nsIFilePicker;
		var filePicker = Components.classes["@mozilla.org/filepicker;1"]
					 .createInstance(Components.interfaces.nsIFilePicker);
		
	/**/	var defaultDirFile = "file://" + MyphraseCCSL.Common.chromeToPath("chrome://myphrase/content/dictionaries/");
	/*	
		var dirService = Components.classes["@mozilla.org/file/directory_service;1"].  
						 getService(Components.interfaces.nsIProperties);   
		var defaultDirFile = dirService.get("Home", Components.interfaces.nsIFile); // returns an nsIFile object   
	*/
	/**/	var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);		  
	/**/	var url = ios.newURI(defaultDirFile, null, null);
	/**/	var defaultDirFile = url.QueryInterface(Components.interfaces.nsIFileURL).file;
		
		filePicker.init(parent, "Save Dictionary File", nsIFilePicker.modeSave);
		filePicker.defaultExtension = "txt";
		filePicker.defaultString = "myphrase_dictionary";
		filePicker.displayDirectory = defaultDirFile;

		filePicker.appendFilters(nsIFilePicker.filterText);
		filePicker.appendFilters(nsIFilePicker.filterAll);

		
		var ret = filePicker.show();
		if (ret == nsIFilePicker.returnOK || ret == nsIFilePicker.returnReplace) {
			var file = filePicker.file;
		} else {
			return;
		}
	 
		var ostream = FileUtils.openSafeFileOutputStream(file)  
		var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].  
						createInstance(Components.interfaces.nsIScriptableUnicodeConverter);  
		converter.charset = "UTF-8";  
		var istream = converter.convertToInputStream(this.outWords);  
		   
		NetUtil.asyncCopy(istream, ostream, function(status) {  
			if (!Components.isSuccessCode(status)) {  
				this.parent.alert("error writing file!");
				return;  
			}    
		});
		return file;
	},
	
	 pickTextFiles: function(parent) {
		var dirService = Components.classes["@mozilla.org/file/directory_service;1"].  
						 getService(Components.interfaces.nsIProperties);   
		var homeDirFile = dirService.get("Home", Components.interfaces.nsIFile); // returns an nsIFile object  
		//var defaultDirFile = homeDirFile.path;

		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		var fp = Components.classes["@mozilla.org/filepicker;1"]
					.createInstance(nsIFilePicker);
					
		fp.init(parent, "Select a File", nsIFilePicker.modeOpenMultiple);
		fp.displayDirectory = homeDirFile;
		//fp.appendFilters("Text or HTML", "*.txt; *.text; *.html; *.htm; *.shtml; *.xhtml; *.xml");
		fp.appendFilters(nsIFilePicker.filterText);
		fp.appendFilters(nsIFilePicker.filterHTML);
		fp.appendFilters(nsIFilePicker.filterAll);

		var res = fp.show();

		if (res == nsIFilePicker.returnOK)
		{
			return fp.files;
		}
	 },

};
